/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Code of the send_protect callback function.
 * See <waaad/security-api.h> for more information.
 */

#include "sec_nosec.h"

/* We just write the buffer "as is" on the connection object */

int sns_send_protect (sec_session_t * session, void ** ext_session, void * data, size_t length)
{
	ssize_t ret = 0;
	size_t sent = 0;
	int  stream = -1;
	
	TRACE_ENTRY("%p %p %p %d", session, ext_session, data, length);
	
	if (!session || !session->conn) {
		TRACE_DEBUG(INFO, "Invalid argument");
		return EINVAL;
	}
	
	while (sent < length) {
		switch (session->proto) {
			case IPPROTO_TCP:
				ret = (*session->cbs.tcp.send_data) (session->conn, ((char *)data) + sent, length - sent);
				break;
				
			case IPPROTO_SCTP:
				ret = (*session->cbs.sctp.send_data) (session->conn, &stream, ((char *)data) + sent, length - sent);
				break;
			
			default:
				TRACE_DEBUG(INFO, "Invalid proto");
				return EINVAL;
		}
				
		if (ret == -1) {
			ret = errno;
			TRACE_DEBUG(INFO, "The send_data callback failed: %s", strerror(ret));
			return ret;
		}
		sent += ret;
	}
	
	return 0;
}
